==> kom%2B%2B-0.2.tar.gz: kom++-0.2/libkom++/text-mass-cache.cc <==
// Fetch-on-demand cache for texts.
//
// $Id: text-mass-cache.cc,v 1.3 1994/07/16 17:47:05 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <iostream.h>

#include "text-mass-cache.h"

#include "get_text_q.h"

// ================================================================ 
//                    Constructors and Destructors                  


Text_mass_cache::Text_mass_cache(Connection *c, int sz)
    : conn(c), size(sz), lru_cache()
{
}
 
Text_mass_cache::Text_mass_cache(const Text_mass_cache &tmc)
    : conn(tmc.conn), size(tmc.size), lru_cache(tmc.lru_cache)
{
}


Text_mass_cache::~Text_mass_cache()
{
    // +++Does the DLList have to be destructed, or does it do that
    // itself, list entries and all?
}


// ================================================================ 
//

//const Text_mass_cache & 
//Text_mass_cache::operator = (const Text_mass_cache &)
//{
//    cerr << "Operator= for class Text_mass_cache is not yet implemented\n";
//    return *this;
//}


//ostream &
//operator <<(ostream &, const Text_mass_cache &)
//{
//}


void
Text_mass_cache::set_size(int sz)
{
    if (sz < 1)
	sz = 1;

    size = sz;
    while (size < lru_cache.length()) {
	lru_remove();
    }
}


Bool
Text_mass_cache::present(Text_no tno)
{
    return (Bool) (find(tno) != 0);
}


Bool
Text_mass_cache::empty()
{
    return (Bool) (lru_cache.empty());
}


void
Text_mass_cache::clear()
{
    while (!empty())
	lru_remove();
}


Connection *
Text_mass_cache::connection() const
{
    return conn;
}

Status
Text_mass_cache::prefetch(Text_no tno)
{
    // +++ NYI
    return st_ok;
}


LyStr *
Text_mass_cache::get(Text_no tno)
{
    Pix  where;
    
    where = find(tno);
    if (where == 0) {
	// Get the text from the server.
        Cache_item          ci;
	get_text_question   get_text_q(conn, tno, 0, 999999999);

	ci.text_no = tno;
	if (get_text_q.receive() == st_ok)
	    ci.text = new LyStr(get_text_q.result());
	else {
	    ci.text = new LyStr("Du fr inte lsa texten");
	    // FIXME! Error handling...
	    // +++
        }

	lru_cache.prepend(ci);
	if (lru_cache.length() > size)
	    lru_remove();
    } else {
	lru_update(where);
    }
    return lru_cache.front().text;
}


void
Text_mass_cache::remove(Text_no tno)
{
    // +++NYI
}


// ================================================================
//                    Private member functions


void
Text_mass_cache::lru_remove()
{
    Cache_item   ci;

    if (!lru_cache.empty()) {
	ci = lru_cache.remove_rear();
	delete ci.text;
    }
}

void
Text_mass_cache::lru_update(Pix where)
{
    if (!lru_cache.empty()){
	Cache_item   ci = lru_cache(where);

	lru_cache.del(where);	
	lru_cache.prepend(ci);
    }
}


Pix
Text_mass_cache::find(Text_no tno)
{
    for (Pix i = lru_cache.first(); i != 0; lru_cache.next(i)) {
	if (lru_cache(i).text_no == tno)
	    return i;
    }

    return 0;
}

==> kom%2B%2B-0.3.tar.gz: kom++-0.3/libkom++/text-mass-cache.cc <==
// Fetch-on-demand cache for texts.
//
// $Id: text-mass-cache.cc,v 1.5 1994/08/01 22:53:45 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <iostream.h>

#include "text-mass-cache.h"

#include "get_text_q.h"

// ================================================================ 
//                    Constructors and Destructors                  


Text_mass_cache::Text_mass_cache(Connection *c, int sz)
    : conn(c), size(sz), lru_cache()
{
}
 
Text_mass_cache::Text_mass_cache(const Text_mass_cache &tmc)
    : conn(tmc.conn), size(tmc.size), lru_cache(tmc.lru_cache)
{
}


Text_mass_cache::~Text_mass_cache()
{
    for (Pix i = lru_cache.first(); i != 0; lru_cache.next(i)) 
	delete lru_cache(i).text;
}


// ================================================================ 
//

//const Text_mass_cache & 
//Text_mass_cache::operator = (const Text_mass_cache &)
//{
//    cerr << "Operator= for class Text_mass_cache is not yet implemented\n";
//    return *this;
//}


//ostream &
//operator <<(ostream &, const Text_mass_cache &)
//{
//}


void
Text_mass_cache::set_size(int sz)
{
    if (sz < 1)
	sz = 1;

    size = sz;
    while (size < lru_cache.length()) {
	lru_remove();
    }
}


Bool
Text_mass_cache::present(Text_no tno)
{
    return (Bool) (find(tno) != 0);
}


Bool
Text_mass_cache::empty()
{
    return (Bool) (lru_cache.empty());
}


void
Text_mass_cache::clear()
{
    while (!empty())
	lru_remove();
}


Connection *
Text_mass_cache::connection() const
{
    return conn;
}

Status
Text_mass_cache::prefetch(Text_no tno)
{
    // +++ NYI
    return st_ok;
}


LyStr *
Text_mass_cache::get(Text_no tno)
{
    Pix  where;
    
    where = find(tno);
    if (where == 0) {
	// Get the text from the server.
        Cache_item          ci;
	get_text_question   get_text_q(conn, tno);

	ci.text_no = tno;
	if (get_text_q.receive() == st_ok)
	    ci.text = new LyStr(get_text_q.result());
	else {
	    ci.text = new LyStr("Du fr inte lsa texten");
	    // FIXME! Error handling...
	    // +++
        }

	lru_cache.prepend(ci);
	if (lru_cache.length() > size)
	    lru_remove();
    } else {
	lru_update(where);
    }
    return lru_cache.front().text;
}


//void
//Text_mass_cache::remove(Text_no tno)
//{
//    // +++NYI
//}


// ================================================================
//                    Private member functions


void
Text_mass_cache::lru_remove()
{
    Cache_item   ci;

    if (!lru_cache.empty()) {
	ci = lru_cache.remove_rear();
	delete ci.text;
    }
}

void
Text_mass_cache::lru_update(Pix where)
{
    if (!lru_cache.empty()){
	Cache_item   ci = lru_cache(where);

	lru_cache.del(where);	
	lru_cache.prepend(ci);
    }
}


Pix
Text_mass_cache::find(Text_no tno)
{
    for (Pix i = lru_cache.first(); i != 0; lru_cache.next(i)) {
	if (lru_cache(i).text_no == tno)
	    return i;
    }

    return 0;
}

Text_mass_cache::Cache_item::Cache_item()
    : text(NULL), text_no(0)
{
}

Text_mass_cache::Cache_item::~Cache_item()
{
    // Note: Cache_items are copied.  It is up to the Text_mass_cache
    // to make sure thate the ``text'' is deleted exactly once.
}
